function [x,y]=linode_2o_seq_n(n,xo,xf,yo);
%
%  [x,y]=linode_2o_seq_n(n,xo,xf,yo);
%
% This script will solve n first-order ODEs using 
% an implicit method that takes advantage of the linearity.
% This implementation solves for each y sequentially.
%
% ODE:  c*dy_i/dx = a*y_i + b
%
% IC:   y_i(xo) = yo_i
%
% inputs:
% n = number of intervals, scalar
% x0 = value of x at initial condition, scalar
% xf = final value of x, scalar
% y0 = value of y at initial condition, vector of length m (m = # of ODEs)
% f_i(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = vector of x values, nx1 vector
% y = matrix of y values, nxm vector
% a graph of y vs x
% a file, 'linode_2o_seq_n_out.txt' containing tabulated data of x and y
%
% example usage:
%
%  [x,y]=linode_2o_seq_n(10,0,2,[1,2]);
%
% Author:  David Keffer 
% original date: October 23, 1998
% last modified date: April 2, 2013
%

%
% define discretization and x vector
%
dx = (xf-xo)/n;
x = zeros(n+1,1);
for i = 1:1:n+1
   x(i) = xo + (i-1)*dx;
end

%
% determine number of ODEs
%
m=max(size(yo));

%
% perform Runge-Kutta steps
%
y = zeros(n+1,m);
y(1,1:m) = yo(1:m);
for k =  2:1:n+1
   AA = zeros(m,m);
   BB = zeros(m,1);
   for i = 1:1:m
      BB(i) = 0.5*dx*(funkeval_b(i,x(k)) + funkeval_b(i,x(k-1)));
      for j = 1:1:m
         AA(i,j) = funkeval_c(i,j,x(k))   - 0.50*dx*funkeval_a(i,j,x(k));
		 term =    funkeval_c(i,j,x(k-1)) + 0.50*dx*funkeval_a(i,j,x(k-1));
		 BB(i) = BB(i) + term*y(k-1,j);
      end
   end
   invAA = inv(AA);
   y(k,1:m) = invAA*BB;
end

%
%  plot
%
close all;
iplot = 1;
if (iplot == 1)
   for i = 1:1:m
      color_index = get_plot_color(i);
      plot (x(:),y(:,i),color_index); 
      hold on;
   end
   hold off;
   xlabel( 'x' ); 
   ylabel ( 'y' );
   legend (int2str([1:m]'));
end

%
%  write result to file 'linode_2o_seq_n_out.txt'
%
fid = fopen('linode_2o_seq_n_out.txt','w');
fprintf(fid,'x  y(1) ... y(m) \n');
for i = 1:1:n+1
   fprintf(fid,'%23.15e ', x(i));
   for j = 1:1:m
      fprintf(fid,'%23.15e ', y(i,j));
   end
   fprintf(fid,' \n');
end
fclose(fid);

%
% enter ODEs in this function
%  c(x)*dy_/dx = A(x)*y_ + b(x);
%
function aout = funkeval_a(i,j,x);
amat = [0 -1; -1 -1];
aout = amat(i,j);

function bout = funkeval_b(i,x);
bvec = [x; 0];
bout = bvec(i);

function cout = funkeval_c(i,j,x);
cmat = [1 0; 0 1];
cout = cmat(i,j);







%
% this little function sets colors for curves in plot
%
function color_index = get_plot_color(i);
if (i == 1) 
   color_index = 'k-';
elseif (i == 2)
   color_index = 'r-';
elseif (i == 3)
   color_index = 'b-';
elseif (i == 4)
   color_index = 'g-';
elseif (i == 5)
   color_index = 'm-';
elseif (i == 6) 
   color_index = 'k:';
elseif (i == 7)
   color_index = 'r:';
elseif (i == 8)
   color_index = 'b:';
elseif (i == 9)
   color_index = 'g:';
elseif (i == 10)
   color_index = 'm:';
else
   color_index = 'k-';
end