function [x,y]=linode_2o_sim_1(n,xo,xf,yo);
%
%  [x,y]=linode_2o_sim_1(n,xo,xf,yo);
%
% This script will solve one linear first-order ODE using 
% an implicit method that takes advantage of the linearity.
% This implementation solves for all y at once using a matrix inversion.
%
% ODE:  dy/dx = f(x,y)
%
% IC:   y(xo) = yo
%
% inputs:
% n = number of intervals, scalar
% x0 = value of x at initial condition, scalar
% xf = final value of x, scalar
% y0 = value of y at initial condition, scalar
% f(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = vector of x values, nx1 vector
% y = vector of y values, nx1 vector
% a graph of y vs x
% a file, 'linode_2o_sim_1_out.txt' containing tabulated data of x and y
%
% example usage:
%
%  [x,y]=linode_2o_sim_1(10,0,2,1);
%
% Author:  David Keffer 
% original date: October 23, 1998
% last modified date: April 2, 2013
%

%
% define discretization and x vector
%
dx = (xf-xo)/n;
x = zeros(n+1,1);
for i = 1:1:n+1
   x(i) = xo + (i-1)*dx;
end
%
% perform Runge-Kutta steps
%
y = zeros(n+1,1);
A = zeros(n+1,n+1);
B = zeros(n+1,1);
%
%  enter initial condition
%
A(1,1) = 1.0;
B(1) = yo;

for i =  2:1:n+1
   B(i) = 0.50*dx*(funkeval_b(x(i)) + funkeval_b(x(i-1)));
   A(i,i) = 1.0 - 0.50*dx*funkeval_a(x(i));
   A(i,i-1) = -1.0 - 0.50*dx*funkeval_a(x(i-1));
end
invA = inv(A);
y = invA*B;
%
%  plot
%
close all;
iplot = 1;
if (iplot == 1)
   plot (x,y,'k-o'), xlabel( 'x' ), ylabel ( 'y' );
end
%
%  write result to file 'linode_2o_sim_1_out.txt'
%
fid = fopen('linode_2o_sim_1_out.txt','w');
fprintf(fid,'x                y \n');
fprintf(fid,'%23.15e %23.15e   \n', [x,y]');
fclose(fid);

%
% enter ODE in these functions
%
%  dydx = a(x)*y(x) + b(x);

function a = funkeval_a(x);
a = -1;

function b = funkeval_b(x);
b = 0;
