function [x,y]=rk4n(n,xo,xf,yo);
%
%  [x,y]=rk4n(n,xo,xf,yo);
%
% This script will solve n first-order ODEs using 
% the classical fourth-order Runge-Kutta method.
%
% ODE:  dy_i/dx = f_i(x,y_j)
%
% IC:   y_i(xo) = yo_i
%
% inputs:
% n = number of intervals, scalar
% x0 = value of x at initial condition, scalar
% xf = final value of x, scalar
% y0 = value of y at initial condition, vector of length m (m = # of ODEs)
% f_i(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = vector of x values, nx1 vector
% y = matrix of y values, nxm vector
% a graph of y vs x
% a file, 'rk4n_out.txt' containing tabulated data of x and y
%
% example usage:
%
%  [x,y]=rk4n(10,0,2,[1,0,2]);
%
% Author:  David Keffer 
% original date: October 23, 1998
% last modified date: April 2, 2013
%

%
% define discretization and x vector
%
dx = (xf-xo)/n;
x = zeros(n+1,1);
for i = 1:1:n+1
   x(i) = xo + (i-1)*dx;
end

%
% determine number of ODEs
%
m=max(size(yo));

%
% perform Runge-Kutta steps
%
y = zeros(n+1,m);
y(1,1:m) = yo(1:m);
dydx = zeros(1,m);
ytemp = zeros(1,m);
k1 = zeros(1,m);
k2 = zeros(1,m);
k3 = zeros(1,m);
k4 = zeros(1,m);
for i =  1:1:n
   x1 = x(i);
   ytemp(1:m) = y(i,1:m);
   k1(1:m) = funkeval(x1,ytemp);
   x2 = x(i) + 0.5*dx;
   ytemp(1:m) = y(i,1:m) + 0.5*dx*k1(1:m);
   k2(1:m) = funkeval(x2,ytemp);
   x3 = x(i) + 0.5*dx;
   ytemp(1:m) = y(i,1:m) + 0.5*dx*k2(1:m);
   k3(1:m) = funkeval(x3,ytemp);
   x4 = x(i) + dx;
   ytemp(1:m) = y(i,1:m) + dx*k3(1:m);
   k4(1:m) = funkeval(x4,ytemp);
   dydx(1:m) = 1.0/6.0*(k1(1:m) + 2.0*k2(1:m) + 2.0*k3(1:m) + k4(1:m));
   y(i+1,1:m) = y(i,1:m) + dx*dydx(1:m);
end

%
%  plot
%
close all;
iplot = 1;
if (iplot == 1)
   for i = 1:1:m
      color_index = get_plot_color(i);
      plot (x(:),y(:,i),color_index); 
      hold on;
   end
   hold off;
   xlabel( 'x' ); 
   ylabel ( 'y' );
   legend (int2str([1:m]'));
end

%
%  write result to file 'rk4n_out.txt'
%
fid = fopen('rk4n_out.txt','w');
fprintf(fid,'x  y(1) ... y(m) \n');
for i = 1:1:n+1
   fprintf(fid,'%23.15e ', x(i));
   for j = 1:1:m
      fprintf(fid,'%23.15e ', y(i,j));
   end
   fprintf(fid,' \n');
end
fclose(fid);

%
% enter ODE in this function
%
function dydx = funkeval(x,y);
dydx(1) = y(2);
dydx(2) = -1.0*y(1) - 0.5*y(2);







%
% this little function sets colors for curves in plot
%
function color_index = get_plot_color(i);
if (i == 1) 
   color_index = 'k-';
elseif (i == 2)
   color_index = 'r-';
elseif (i == 3)
   color_index = 'b-';
elseif (i == 4)
   color_index = 'g-';
elseif (i == 5)
   color_index = 'm-';
elseif (i == 6) 
   color_index = 'k:';
elseif (i == 7)
   color_index = 'r:';
elseif (i == 8)
   color_index = 'b:';
elseif (i == 9)
   color_index = 'g:';
elseif (i == 10)
   color_index = 'm:';
else
   color_index = 'k-';
end