function [x,y]=rk41(n,xo,xf,yo);
%
%  [x,y]=rk41(n,xo,xf,yo);
%
% This script will solve one first-order ODE using 
% the Classical Fourth-order Runge-Kutta method.
%
% ODE:  dy/dx = f(x,y)
%
% IC:   y(xo) = yo
%
% inputs:
% n = number of intervals, scalar
% x0 = value of x at initial condition, scalar
% xf = final value of x, scalar
% y0 = value of y at initial condition, scalar
% f(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = vector of x values, nx1 vector
% y = vector of y values, nx1 vector
% a graph of y vs x
% a file, 'rk41_out.txt' containing tabulated data of x and y
%
% example usage:
%
%  [x,y]=rk41(10,0,2,1);
%
% Author:  David Keffer 
% original date: October 23, 1998
% last modified date: April 2, 2013
%

%
% define discretization and x vector
%
dx = (xf-xo)/n;
x = zeros(n+1,1);
for i = 1:1:n+1
   x(i) = xo + (i-1)*dx;
end
%
% perform Runge-Kutta steps
%
y = zeros(n+1,1);
y(1) = yo;
for i =  1:1:n
   x1 = x(i);
   y1 = y(i);
   k1 = funkeval(x1,y1);
   x2 = x(i) + 0.5*dx;
   y2 = y(i) + 0.5*dx*k1;
   k2 = funkeval(x2,y2);
   x3 = x(i) + 0.5*dx;
   y3 = y(i) + 0.5*dx*k2;
   k3 = funkeval(x3,y3);
   x4 = x(i) + dx;
   y4 = y(i) + dx*k3;
   k4 = funkeval(x4,y4);
   dydx = 1.0/6.0*(k1 + 2.0*k2 + 2.0*k3 + k4);
   y(i+1) = y(i) + dx*dydx;
end
%
%  plot
%
close all;
iplot = 1;
if (iplot == 1)
   plot (x,y,'k-o'), xlabel( 'x' ), ylabel ( 'y' );
end
%
%  write result to file 'rk41_out.txt'
%
fid = fopen('rk41_out.txt','w');
fprintf(fid,'x                y \n');
fprintf(fid,'%23.15e %23.15e   \n', [x,y]');
fclose(fid);

%
% enter ODE in this function
%
function dydx = funkeval(x,y);
dydx = -1.0*y;
